@@ -27,6 +27,12 @@ else:
 from .data import DATA_DIR, DEFAULT_YEAR, URL, WAIT_TIME, get_cookie
 
 
+def _open_page(page: str) -> None:
+    """Open the page if the user hasn't opted out"""
+    if not (DATA_DIR / ".nobrowser").exists():
+        webbrowser.open(page)
+
+
 def _make(folder: pathlib.Path) -> None:
     """Create folder if it doesn't exist."""
     if not folder.exists():
@@ -34,6 +40,7 @@ def _make(folder: pathlib.Path) -> None:
 
 
 def _pretty_print(message: str) -> None:
+    """Analyse and print message"""
     if message.startswith("That's the"):
         print(GREEN + message + RESET)
     elif message.startswith("You don't"):
@@ -65,14 +72,23 @@ def fetch(
             print(YELLOW + "Waiting for puzzle unlock..." + RESET)
             time.sleep((unlock - now).total_seconds())
             print(GREEN + "Fetching input!" + RESET)
+            _open_page(URL.format(day=day, year=year))
         resp = requests.get(
             URL.format(day=day, year=year) + "/input", cookies=get_cookie()
         )
         if not resp.ok:
+            if resp.status_code == 400:
+                token_file = DATA_DIR / "token.txt"
+                token = input(
+                    "Your token has expired. Please enter your new token\n>>> "
+                )
+                token_file.write_text(token)
+                return fetch(day, year)
             raise ValueError("Received bad response")
         # Note to star: May consider rstrip instead -- I don't know if AoC will ever
-        # publish input that has significant whitespace at the beginning though.
-        data = resp.text.strip()
+        # publish input that has significant whitespace at the beginning though. --
+        # should now be fixed: stripping newlines only
+        data = resp.text.strip("\n")
         input_path.write_text(data)
         return data
 
@@ -134,6 +150,13 @@ def submit(day: int, part: int, answer: typing.Any, year: int = DEFAULT_YEAR) ->
             data={"level": part_, "answer": answer_},
         )
         if not resp.ok:
+            if resp.status_code == 400:
+                token_file = DATA_DIR / "token.txt"
+                token = input(
+                    "Your token has expired. Please enter your new token\n>>> "
+                )
+                token_file.write_text(token)
+                return submit(day, part, answer, year)
             raise ValueError("Received bad response")
 
         msg: str = Soup(resp.text, "html.parser").article.text
@@ -151,7 +174,9 @@ def submit(day: int, part: int, answer: typing.Any, year: int = DEFAULT_YEAR) ->
     if msg.startswith("That's the"):
         solution_file.write_text(answer_)
         if part == 1:
-            webbrowser.open(resp.url)  # open part 2 in the user's browser
+            if not resp.url.endswith("#part2"):
+                resp.url += "#part2"  # scroll to part 2
+            _open_page(resp.url)  # open part 2 in the user's browser
 
     # Cache submission
     solutions[part_][answer_] = msg
